@@ -10,7 +10,6 @@ from astropy.io import fits
 from astropy.nddata import StdDevUncertainty, VarianceUncertainty, InverseVariance
 from astropy.time import Time
 from astropy.wcs import WCS
-import asdf
 from gwcs.wcstools import grid_from_bounding_box
 
 from ...spectra import Spectrum1D, SpectrumList
@@ -255,6 +254,7 @@ def jwst_x1d_miri_mrs_loader(input, missing="raise", **kwargs):
     SpectrumList
         A list of the spectra that are contained in all the files.
     """
+
     # If input is a list, go read each file. If directory, glob-expand
     # list of file names.
     if not isinstance(input, (list, tuple)):
@@ -317,6 +317,12 @@ def _jwst_spec1d_loader(file_obj, extname='EXTRACT1D', **kwargs):
             if hdu.name != extname:
                 continue
 
+            # Correct some known bad unit strings before reading the table
+            bad_units = {"(MJy/sr)^2": "MJy2 sr-2"}
+            for c in hdu.columns:
+                if c.unit in bad_units:
+                    c.unit = bad_units[c.unit]
+
             data = Table.read(hdu)
 
             wavelength = Quantity(data["WAVELENGTH"])
@@ -351,7 +357,7 @@ def _jwst_spec1d_loader(file_obj, extname='EXTRACT1D', **kwargs):
                     uncertainty = None
             elif srctype == "EXTENDED":
                 flux = Quantity(data["SURF_BRIGHT"])
-                uncertainty = StdDevUncertainty(hdu.data["SB_ERROR"])
+                uncertainty = StdDevUncertainty(data["SB_ERROR"])
             else:
                 raise RuntimeError(f"Keyword SRCTYPE is {srctype}.  It should "
                                    "be 'POINT' or 'EXTENDED'. Can't decide between `flux` and "
@@ -433,11 +439,13 @@ def _jwst_s2d_loader(filename, **kwargs):
     SpectrumList
         The spectra contained in the file.
     """
+    from stdatamodels import asdf_in_fits
+
     spectra = []
     slits = None
 
     # Get a list of GWCS objects from the slits
-    with asdf.open(filename) as af:
+    with asdf_in_fits.open(filename) as af:
         # Slits can be listed under "slits", "products" or "exposures"
         if "products" in af.tree:
             slits = "products"
@@ -554,10 +562,12 @@ def _jwst_s3d_loader(filename, **kwargs):
     SpectrumList
         The spectra contained in the file.
     """
+    from stdatamodels import asdf_in_fits
+
     spectra = []
 
     # Get a list of GWCS objects from the slits
-    with asdf.open(filename) as af:
+    with asdf_in_fits.open(filename) as af:
         wcslist = [af.tree["meta"]["wcs"]]
 
     with fits.open(filename, memmap=False) as hdulist:
