@@ -35,7 +35,7 @@ class Spectrum1D(OneDSpectrumMixin, NDCube, NDIOMixin, NDArithmeticMixin):
 
     Parameters
     ----------
-    flux : `~astropy.units.Quantity` or `~astropy.nddata.NDData`-like
+    flux : `~astropy.units.Quantity`
         The flux data for this spectrum. This can be a simple `~astropy.units.Quantity`,
         or an existing `~Spectrum1D` or `~ndcube.NDCube` object.
     spectral_axis : `~astropy.units.Quantity` or `~specutils.SpectralAxis`
@@ -296,6 +296,11 @@ class Spectrum1D(OneDSpectrumMixin, NDCube, NDIOMixin, NDArithmeticMixin):
                 doppler_rest=rest_value,
                 doppler_convention=velocity_convention)
 
+        # make sure that spectral axis is strictly increasing or decreasing,
+        # raise an error if not.
+        if not self._check_strictly_increasing_decreasing():
+            raise ValueError('Spectral axis must be strictly increasing or decreasing.')
+
         if hasattr(self, 'uncertainty') and self.uncertainty is not None:
             if not flux.shape == self.uncertainty.array.shape:
                 raise ValueError(
@@ -584,6 +589,10 @@ class Spectrum1D(OneDSpectrumMixin, NDCube, NDIOMixin, NDArithmeticMixin):
     def shape(self):
         return self.flux.shape
 
+    @property
+    def spectral_axis_direction(self):
+        return self._spectral_axis_direction
+
     @property
     def redshift(self):
         """
