@@ -357,6 +357,18 @@ def test_create_with_uncertainty():
         Spectrum1D(spectral_axis=wavelengths*u.um, flux=flux, uncertainty=uncertainty)
 
 
+@pytest.mark.parametrize("flux_unit", ["adu", "ct/s", "count"])
+def test_flux_unit_io_roundtrip(tmp_path, flux_unit):
+    # regression test for https://github.com/astropy/specutils/pull/1018
+    fname = str(tmp_path / 'flux_unit_io_roundtrip.fits')
+    sp = Spectrum1D(flux=np.ones(11) * u.Unit(flux_unit),
+                    spectral_axis=np.arange(1, 12) * u.Unit('Hz'))
+    sp.write(fname, overwrite=True)
+
+    sp_load = Spectrum1D.read(fname)
+    assert sp_load.flux.unit == sp.flux.unit
+
+
 @pytest.mark.filterwarnings('ignore::astropy.io.fits.verify.VerifyWarning')
 @remote_access([{'id': '1481190', 'filename': 'L5g_0355+11_Cruz09.fits'}])
 def test_read_linear_solution(remote_data_path):
@@ -495,3 +507,31 @@ def test_collapse_flux():
     median_spec = spec.mean(axis = 0)
     assert isinstance(median_spec, Spectrum1D)
     assert np.all(median_spec.flux == [2, 8, 9] * u.Jy)
+
+
+def test_unsorted_spectral_axis_fails():
+    """
+    Test that creating a Spectrum1D fails if spectral axis isn't strictly
+    ascending or descending.
+    """
+    wave = [2, 1, 3] * u.nm
+    flux = [5, 5, 5] * u.Jy
+
+    with pytest.raises(ValueError, match='Spectral axis must be strictly increasing or decreasing.'):
+        Spectrum1D(spectral_axis=wave, flux=flux)
+
+
+def test_spectral_axis_direction():
+    """
+    Test that the spec1d.spectral_axis_direction attribute correctly reflects
+    if the spectral axis in Spectrum1D is increasing or decreasing.
+    """
+    flux = [5, 5, 5] * u.Jy
+
+    wave = [1, 2, 3] * u.nm
+    spec1d = Spectrum1D(spectral_axis=wave, flux=flux)
+    assert spec1d.spectral_axis_direction == 'increasing'
+
+    wave = [3, 2, 1] * u.nm
+    spec1d = Spectrum1D(spectral_axis=wave, flux=flux)
+    assert spec1d.spectral_axis_direction == 'decreasing'
